/*
 * This software is provided "as is"; redistribution and modification
 * is permitted, provided that the following disclaimer is retained.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * In no event shall the authors or contributors be liable for any
 * direct, indirect, incidental, special, exemplary, or consequential
 * damages (including, but not limited to, procurement of substitute
 * goods or services; loss of use, data, or profits; or business
 * interruption) however caused and on any theory of liability, whether
 * in contract, strict liability, or tort (including negligence or
 * otherwise) arising in any way out of the use of this software, even
 * if advised of the possibility of such damage.
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sched.h>
#include <sys/stat.h>
#include <sys/time.h>

double
gettime()
{
  timeval tv;
  gettimeofday (&tv, 0);

  return tv.tv_sec + tv.tv_usec / 1000000.0;
}

int
main()
{
  if (getenv ("FS_ONE_CPU"))
    {
      cpu_set_t cpus;
      CPU_ZERO (&cpus);
      CPU_SET (0, &cpus);
      sched_setaffinity (getpid(), sizeof (cpu_set_t), &cpus);
    }

  int same = 0, other = 0;

  for (int i = 0; i < 10000; i++)
    {
      char buffer[64];
      if (readlink ("cpu", buffer, 64))
        {
          int cpu = sched_getcpu();
          if (cpu == atoi (buffer))
            same++;
          else
            other++;
        }
    }
  printf ("same  cpu (%%) %8.2f\n", same * 100.0 / (same + other));
  printf ("other cpu (%%) %8.2f\n", other * 100.0 / (same + other));
  printf ("----------------------\n");

  double t_start = gettime();
  for (int i = 0; i < 10000; i++)
    {
      struct stat st;

      stat ("cpu", &st);
    }
  double t_end = gettime();
  printf ("stat calls/s  %8.2f\n", 10000 / (t_end - t_start));
  return 0;
}
