/*
 * This software is provided "as is"; redistribution and modification
 * is permitted, provided that the following disclaimer is retained.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * In no event shall the authors or contributors be liable for any
 * direct, indirect, incidental, special, exemplary, or consequential
 * damages (including, but not limited to, procurement of substitute
 * goods or services; loss of use, data, or profits; or business
 * interruption) however caused and on any theory of liability, whether
 * in contract, strict liability, or tort (including negligence or
 * otherwise) arising in any way out of the use of this software, even
 * if advised of the possibility of such damage.
 */

#define FUSE_USE_VERSION 26

#include <fuse.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sched.h>
#include <stdlib.h>

static int
fs_readdir (const char *path, void *buf, fuse_fill_dir_t filler,
            off_t offset, struct fuse_file_info *fi)
{
  (void) offset;
  (void) fi;

  filler (buf, ".", NULL, 0);
  filler (buf, "..", NULL, 0);
  filler (buf, "cpu", NULL, 0);
  return 0;
}

static int
fs_getattr (const char *path, struct stat *stbuf)
{
  if (strcmp (path, "/") == 0)
    {
      memset (stbuf, 0, sizeof (struct stat));
      stbuf->st_mode = 0755 | S_IFDIR;
      stbuf->st_uid  = getuid();
      stbuf->st_gid  = getgid();
      return 0;
    }
  if (strcmp (path, "/cpu") == 0)
    {
      memset (stbuf, 0, sizeof (struct stat));
      stbuf->st_mode = 0755 | S_IFLNK;
      stbuf->st_uid  = getuid();
      stbuf->st_gid  = getgid();
      return 0;
    }
  return 0;
}

static int
fs_readlink (const char *path, char *buffer, size_t size)
{
  if (strcmp (path, "/cpu") == 0)
    {
      int cpu = sched_getcpu();
      snprintf (buffer, size, "%d", cpu);
      return 0;
    }
  else
    return -EINVAL;
}



int
main (int argc, char **argv)
{
  if (getenv ("FS_ONE_CPU"))
    {
      cpu_set_t cpus;
      CPU_ZERO (&cpus);
      CPU_SET (0, &cpus);
      sched_setaffinity (getpid(), sizeof (cpu_set_t), &cpus);
    }

  static struct fuse_operations fs_oper = { NULL, };

  fs_oper.readdir = fs_readdir;
  fs_oper.getattr = fs_getattr;
  fs_oper.readlink = fs_readlink;

  return fuse_main (argc, argv, &fs_oper, NULL);
}
